package cd.casic.ci.process.engine.worker.afl;


import cd.casic.ci.process.common.WorkAtom;
import cd.casic.ci.process.dto.req.resource.ResourceQueryReq;
import cd.casic.ci.process.dto.resp.resource.ResourceFindResp;
import cd.casic.ci.process.dto.resp.taskResource.TaskResourceFindResp;
import cd.casic.ci.process.engine.constant.DIYImageExecuteCommandConstant;
import cd.casic.ci.process.engine.constant.PipelineGlobalVariableConstant;
import cd.casic.ci.process.engine.constant.TestCaseGenerationConstant;
import cd.casic.ci.process.engine.runContext.TaskRunContext;
import cd.casic.ci.process.engine.worker.base.DockerWorker;
import cd.casic.ci.process.engine.worker.base.SshWorker;
import cd.casic.ci.process.process.dataObject.pipeline.PipPipeline;
import cd.casic.ci.process.process.dataObject.resource.PipResourceMachine;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.util.CryptogramUtil;
import cd.casic.ci.process.util.SftpUploadUtil;
import cd.casic.module.execute.docker.dataobject.dto.DockerEndpointDo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.util.Map;

@WorkAtom(taskType = "TEST_CASE_GENERATION")
@Slf4j
public class TestCaseGenerationWorker extends DockerWorker {
    @Override
    public void execute(TaskRunContext context) {
        int statusCode = -1;
        Map<String, Object> globalVariables = context.getGlobalVariables();
        String workDir = globalVariables.get(PipelineGlobalVariableConstant.AFL_DOCKER_WORK_DIR_KEY) instanceof String ? ((String) globalVariables.get(PipelineGlobalVariableConstant.AFL_DOCKER_WORK_DIR_KEY)) : null;
        if (context.getContextDef() instanceof PipTask taskDef) {
            String seedPath= workDir+File.separator+PipelineGlobalVariableConstant.AFL_DOCKER_SEED;
            Map<String, Object> taskProperties = taskDef.getTaskProperties();
            Object caseType = taskProperties.get(TestCaseGenerationConstant.CASE_TYPE_KEY);
            String binaryName = taskProperties.get("binaryName").toString();
            String managerId = taskProperties.get("managerId") instanceof String ? ((String) taskProperties.get("managerId")) : null;
            ResourceFindResp resourceById = resourceManagerService.findResourceById(managerId);
            String machineId = resourceById.getMachineId();
            String dockerId = resourceById.getDockerId();
            if (StringUtils.isEmpty(machineId)||StringUtils.isEmpty(dockerId)) {
                append(context,"该资源不支持docker或者ssh");
            }
            ResourceQueryReq req = new ResourceQueryReq();
            req.setId(machineId);
            req.setType("machine");
            TaskResourceFindResp machineQuery = resourceManagerService.findResourceListByType(req);
            req.setId(dockerId);
            req.setType("docker");
            TaskResourceFindResp dockerQuery = resourceManagerService.findResourceListByType(req);
            PipResourceMachine machineInfo = machineQuery.getResourceMachineList().get(0);
            DockerEndpointDo dockerInfo = dockerQuery.getDockerEndpointList().get(0);
            //如果machineId为0，则说明该节点没有配置机器，则使用开始节点的机器
            if (TestCaseGenerationConstant.CASE_TYPE_AI.equals(caseType)) {
                // 容器内部test 目录, 获取编译后的文件
                String runScript = "docker run -v "+ PipelineGlobalVariableConstant.AFL_VOLUME_WORK_DIR_PREFIX +":"+PipelineGlobalVariableConstant.AFL_DOCKER_BASE_DIR+"  -it aflplusplus bash\n" // 目前测试用例ai生成只有这个路径能跑
                + "cd "+PipelineGlobalVariableConstant.AFL_DOCKER_BASE_DIR+"\n" +
                        "mkdir -p "+seedPath +"\n"+
                        "cd "+seedPath +"\n"+
                        "PYTHONPATH=CaseGenerator/src python3 CaseGenerator/src/CaseGenerator/main.py --work-dir "+workDir+" --binary "+workDir+File.separator+binaryName+" --output-dir "+seedPath+" --count 100";
                    //将节点的配置信息反编译成对象
                    log.info("测试用例生成-AI生成，实际执行命令:{}" , runScript);
                    //获取机器
                dockerRun(runScript,dockerInfo,context);
            } else {
                // 文件上传
                String filePath = taskProperties.get(TestCaseGenerationConstant.SEED_SOURCE) instanceof String ? ((String) taskProperties.get(TestCaseGenerationConstant.SEED_SOURCE)) : null;
                if (StringUtils.isEmpty(filePath)) {
                    append(context,"缺少文件");
                    toBadEnding();
                }
                File file = new File(filePath);
                String seedTarget = PipelineGlobalVariableConstant.AFL_VOLUME_WORK_DIR_PREFIX+workDir+File.separator+seedPath;
                // 将文件上传到服务器的 目录底下
                log.info("测试用例选用上传模式,种子文件路径：{},种子上传路径:{}",filePath,seedTarget);
                try {
                    SftpUploadUtil.uploadFileViaSftp(
                            machineInfo.getMachineHost()
                            ,Integer.valueOf(machineInfo.getSshPort()),machineInfo.getUsername(),machineInfo.getPassword(),"",filePath,seedTarget,file.getName());
                } catch (SftpUploadUtil.SftpUploadException e) {
                    append(context,"seed文件上传失败");
                    log.error("seed文件上传失败",e);
                }
            }
            globalVariables.put(PipelineGlobalVariableConstant.AFL_DOCKER_SEED_PATH_KEY,seedPath);
        }
    }
}
